@@ -580,7 +580,7 @@ def fast_read_image_roi(filename, orig_rect, ImreadModes=cv2.IMREAD_COLOR, norma
     return roi_image
 
 
-def resize_scale_image(image, size: int, use_length=True):
+def resize_scale_image(image, size: int, use_length=True, interpolation=cv2.INTER_LINEAR):
     """
     按照长/短边进行等比例缩放
     :param image:
@@ -591,11 +591,11 @@ def resize_scale_image(image, size: int, use_length=True):
     height, width = image.shape[:2]
     r = [size / width, size / height]
     r = min(r) if use_length else max(r)
-    dimage = cv2.resize(image, dsize=(int(width * r), int(height * r)))
+    dimage = cv2.resize(image, dsize=(int(width * r), int(height * r)), interpolation=interpolation)
     return dimage
 
 
-def resize_image_padding(image, size: Tuple[int, int], use_length=True, color=(0, 0, 0)):
+def resize_image_padding(image, size: Tuple, use_length=True, color=(0, 0, 0), interpolation=cv2.INTER_LINEAR):
     """
     按照长/短边进行等比例缩放，短边会进行填充,长边会被裁剪，避免出现形变
     :param image:
@@ -606,12 +606,12 @@ def resize_image_padding(image, size: Tuple[int, int], use_length=True, color=(0
     """
     height, width = image.shape[:2]
     _size = max(size) if use_length else min(size)
-    image = resize_scale_image(image, size=int(_size), use_length=use_length)
+    image = resize_scale_image(image, size=int(_size), use_length=use_length, interpolation=interpolation)
     image = center_crop_padding(image, crop_size=size, color=color)
     return image
 
 
-def resize_image(image, size: Tuple[int, int]):
+def resize_image(image, size: Tuple[int, int], interpolation=cv2.INTER_LINEAR):
     """
     tf.image.resize_images(images,size),images=[batch, height, width, channels],size=(new_height, new_width)
     cv2.resize(image, dsize=(width, height)),与image.shape相反
@@ -630,7 +630,7 @@ def resize_image(image, size: Tuple[int, int]):
         resize_height = int(height * resize_width / width)
     elif resize_width is None:
         resize_width = int(width * resize_height / height)
-    image = cv2.resize(image, dsize=(int(resize_width), int(resize_height)))
+    image = cv2.resize(image, dsize=(int(resize_width), int(resize_height)), interpolation=interpolation)
     return image
 
 
@@ -926,7 +926,8 @@ def show_image_boxes(title, image, boxes_list, color=(0, 0, 255), delay=0):
     return image
 
 
-def draw_image_bboxes_text(rgb_image, boxes, boxes_name, color, thickness=2, drawType="custom", top=True):
+def draw_image_bboxes_text(rgb_image, boxes, boxes_name, color=(255, 0, 0), thickness=2, fontScale=0.5,
+                           drawType="custom", top=True):
     """
     :param boxes_name:
     :param bgr_image: bgr image
@@ -943,12 +944,12 @@ def draw_image_bboxes_text(rgb_image, boxes, boxes_name, color, thickness=2, dra
         # cv2.putText(bgr_image, name, (crop_type[0], crop_type[1]), cv2.FONT_HERSHEY_SIMPLEX, 0.8, (0, 0, 255), thickness=2)
         # cv2.rectangle(bgr_image, (crop_type[0], crop_type[1]), (crop_type[2], crop_type[3]), color, 2, 8, 0)
         # cv2.putText(bgr_image, str(name), (crop_type[0], crop_type[1]), cv2.FONT_HERSHEY_SIMPLEX, 0.8, color, thickness=2)
-        custom_bbox_line(rgb_image, box, color, name, thickness, drawType, top)
+        custom_bbox_line(rgb_image, box, color, name, thickness, fontScale, drawType, top)
     return rgb_image
 
 
-def draw_image_bboxes_labels_text(rgb_image, boxes, labels, boxes_name=None, color=None, thickness=2, drawType="custom",
-                                  top=True):
+def draw_image_bboxes_labels_text(rgb_image, boxes, labels, boxes_name=None, color=None, thickness=2, fontScale=0.5,
+                                  drawType="custom", top=True):
     """
     :param rgb_image:
     :param boxes:
@@ -966,7 +967,7 @@ def draw_image_bboxes_labels_text(rgb_image, boxes, labels, boxes_name=None, col
     for label, box, name in zip(labels, boxes, boxes_name):
         box = [int(b) for b in box]
         color_ = color if color else color_map[int(label) + 1]
-        custom_bbox_line(rgb_image, box, color_, str(name), thickness, drawType, top)
+        custom_bbox_line(rgb_image, box, color_, str(name), thickness, fontScale, drawType, top)
     return rgb_image
 
 
@@ -1019,9 +1020,40 @@ def show_image_rects_text(title, rgb_image, rects, rects_name, color=None, drawT
     return rgb_image
 
 
-def draw_image_detection_rects(rgb_image, rects, probs, labels, class_name=None):
+def draw_image_bboxes_labels(rgb_image, bboxes, labels, class_name=None, color=None, thickness=2, fontScale=0.8):
+    """
+    :param rgb_image:
+    :param bboxes:  [[x1,y1,x2,y2],[x1,y1,x2,y2]]
+    :param labels:
+    :return:
+    """
+    if isinstance(labels, np.ndarray): labels = labels.astype(np.int32).reshape(-1).tolist()
+    for label, box in zip(labels, bboxes):
+        color_ = color if color else color_map[int(label) + 1]
+        box = [int(b) for b in box]
+        if class_name: label = class_name[int(label)]
+        rgb_image = custom_bbox_line(rgb_image, box, color_, str(label), thickness=thickness,
+                                     fontScale=fontScale, drawType="custom")
+    return rgb_image
+
+
+def draw_image_rects_labels(rgb_image, rects, labels, class_name=None, color=None, thickness=2, fontScale=0.5):
+    """
+    :param rgb_image:
+    :param rects:
+    :param labels:
+    :return:
+    """
     bboxes = rects2bboxes(rects)
-    rgb_image = draw_image_detection_bboxes(rgb_image, bboxes, probs, labels, class_name)
+    rgb_image = draw_image_bboxes_labels(rgb_image, bboxes, labels, class_name=class_name, color=color,
+                                         thickness=thickness, fontScale=fontScale)
+    return rgb_image
+
+
+def draw_image_detection_rects(rgb_image, rects, probs, labels, class_name=None, thickness=2, fontScale=0.5):
+    bboxes = rects2bboxes(rects)
+    rgb_image = draw_image_detection_bboxes(rgb_image, bboxes, probs, labels, class_name,
+                                            thickness=thickness, fontScale=fontScale)
     return rgb_image
 
 
@@ -1039,13 +1071,7 @@ def show_image_detection_rects(title, rgb_image, rects, probs, lables, color=Non
     return rgb_image
 
 
-def flat_data(data):
-    if isinstance(data, np.ndarray):
-        data = data.reshape(-1).tolist()
-    return data
-
-
-def draw_image_detection_bboxes(rgb_image, bboxes, probs, labels, class_name=None, thickness=2):
+def draw_image_detection_bboxes(rgb_image, bboxes, probs, labels, class_name=None, thickness=2, fontScale=0.5):
     """
     :param title:
     :param rgb_image:
@@ -1062,12 +1088,14 @@ def draw_image_detection_bboxes(rgb_image, bboxes, probs, labels, class_name=Non
         if class_name:
             label = class_name[int(label)]
         boxes_name = "{}:{:3.3f}".format(label, prob)
-        custom_bbox_line(rgb_image, box, color, boxes_name, thickness, drawType="custom")
+        custom_bbox_line(rgb_image, box, color, boxes_name, thickness=thickness, fontScale=fontScale, drawType="custom")
     return rgb_image
 
 
-def show_image_detection_bboxes(title, rgb_image, bboxes, probs, labels, class_name=None, delay=0):
-    rgb_image = draw_image_detection_bboxes(rgb_image, bboxes, probs, labels, class_name)
+def show_image_detection_bboxes(title, rgb_image, bboxes, probs, labels, class_name=None, thickness=2, fontScale=0.5,
+                                delay=0):
+    rgb_image = draw_image_detection_bboxes(rgb_image, bboxes, probs, labels, class_name,
+                                            thickness=thickness, fontScale=fontScale)
     cv_show_image(title, rgb_image, delay=delay)
     return rgb_image
 
@@ -1104,7 +1132,7 @@ def draw_dt_gt_dets(image, dt_boxes, dt_label, gt_boxes, gt_label, vis_diff=Fals
     return image
 
 
-def custom_bbox_line(image, bbox, color, name, thickness=2, drawType="custom", top=True):
+def custom_bbox_line(image, bbox, color, name, thickness=2, fontScale=0.5, drawType="custom", top=True):
     """
     :param image:
     :param bbox:
@@ -1114,7 +1142,7 @@ def custom_bbox_line(image, bbox, color, name, thickness=2, drawType="custom", t
     :param top:
     :return:
     """
-    fontScale = 0.5
+    # fontScale = 0.5
     if not name:
         drawType = "simple"
     if drawType == "simple":
@@ -2299,7 +2327,7 @@ def get_mask_boundrect(mask, binarize=False, shift=0):
     return [xmin, ymin, xmax, ymax]
 
 
-def find_mask_contours(mask):
+def find_mask_contours(mask, mode=cv2.RETR_LIST, method=cv2.CHAIN_APPROX_NONE):
     """
     寻找一个二值Mask图像的轮廓
     cv2.findContours(mask，mode, method)
@@ -2317,7 +2345,7 @@ def find_mask_contours(mask):
     :param mask: 输入mask必须是二值Mask图像，注意黑色表示背景，白色表示物体，即在黑色背景里寻找白色物体的轮廓
     :return: contours List[np.ndarray(num_point,2)] 返回二值Mask图像的轮廓
     """
-    contours, hierarchy = cv2.findContours(mask, mode=cv2.RETR_LIST, method=cv2.CHAIN_APPROX_SIMPLE)
+    contours, hierarchy = cv2.findContours(mask, mode=mode, method=method)
     contours = [c.reshape(-1, 2) for c in contours]
     return contours
 
@@ -2511,7 +2539,7 @@ def image_composite(image: np.ndarray, alpha: np.ndarray, bg_img=(219, 142, 67))
 
 def frames2gif_by_imageio(frames, gif_file="test.gif", fps=2, loop=0, use_rgb=False):
     """
-    pip install imageio
+    pip install imageio 文件大，但质量较好
     :param frames:
     :param gif_file: 输出的GIF图的路径
     :param fps: 刷新频率
@@ -2531,7 +2559,7 @@ def frames2gif_by_imageio(frames, gif_file="test.gif", fps=2, loop=0, use_rgb=Fa
 def frames2gif_by_pil(frames, gif_file="test.gif", fps=2, loop=0, use_rgb=False):
     """
     https://pillow.readthedocs.io/en/stable/handbook/image-file-formats.html#gif
-    PS :loop=0表示无限循环，loop=n表示循环n+1次，如要循环一次，需要去掉loop参数(很奇葩吧！！！)
+    PS :loop=0表示无限循环，loop=n表示循环n+1次，如要循环一次，需要去掉loop参数(很奇葩吧！！！)，文件小，但质量较差
     :param frames:
     :param gif_file: 输出的GIF图的路径
     :param fps: 刷新频率
@@ -2549,7 +2577,7 @@ def frames2gif_by_pil(frames, gif_file="test.gif", fps=2, loop=0, use_rgb=False)
                        optimize=False, loop=loop)
 
 
-def image_file_list2gif(file_list, size=None, gif_file="test.gif", fps=4, loop=0):
+def image_file_list2gif(file_list, size=None, gif_file="test.gif", fps=4, loop=0, use_pil=True):
     """
     pip install imageio
     uri：合成后的gif动图的名字，可以随意更改。
@@ -2564,16 +2592,23 @@ def image_file_list2gif(file_list, size=None, gif_file="test.gif", fps=4, loop=0
     :param gif_file: 输出的GIF图的路径
     :param fps: 刷新频率
     :param loop: 循环次数
+    :param use_pil: True使用PIL库生成GIF图，文件小，但质量较差
+                    False使用imageio库生成GIF图，文件大，但质量较好
     :return:
     """
     frames = []
     for file in file_list:
         bgr = cv2.imread(file)
         image = cv2.cvtColor(bgr, cv2.COLOR_BGR2RGB)
-        image = resize_image(image, size=size)
+        if size:
+            image = resize_image_padding(image, size=size)
+        else:
+            image = resize_image(image, size=size)
         frames.append(image)
-    frames2gif_by_pil(frames, gif_file, fps=fps, loop=loop)
-    # images2gif_by_imageio(frames,gif_file, fps=fps, loop=loop)
+    if use_pil:
+        frames2gif_by_pil(frames, gif_file, fps=fps, loop=loop)
+    else:
+        frames2gif_by_imageio(frames, gif_file, fps=fps, loop=loop)
 
 
 def get_video_capture(video_path, width=None, height=None, fps=None):
@@ -2625,7 +2660,7 @@ class CVVideo():
     def start_capture(self, video_path, save_video=None, detect_freq=1):
         """
         start capture video
-        :param video_path: *.avi,*.mp4,...
+        :param video_path: *.avi,*.mp4,...or camera id
         :param save_video: *.avi
         :param detect_freq:
         :return:
@@ -2640,7 +2675,7 @@ class CVVideo():
         while True:
             if count % detect_freq == 0:
                 # 设置抽帧的位置
-                video_cap.set(cv2.CAP_PROP_POS_FRAMES, count)
+                if isinstance(video_file, str): video_cap.set(cv2.CAP_PROP_POS_FRAMES, count)
                 isSuccess, frame = video_cap.read()
                 if not isSuccess:
                     break
